# encoding: utf-8

require 'one_apm/collector/sampler'

module OneApm
  module Collector
    module Samplers
      class CpuSampler < OneApm::Collector::Sampler
        attr_reader :last_time

        named :cpu

        def initialize
          @processor_count = OneApm::Support::SystemInfo.num_logical_processors
          poll
        end

        def set_poller poller
          @poller = poller
        end
        
        def poll
          now = Time.now
          @poller.pid = Process.pid if @poller
          t = Process.times

          if @last_time
            elapsed = now - @last_time
            return if elapsed < 1 # Causing some kind of math underflow

            usertime = t.utime - @last_utime
            systemtime = t.stime - @last_stime

            if systemtime >= 0
              OneApm::Manager.record_metric("CPU/System Time", systemtime)
              OneApm::Manager.record_metric("CPU/System/Utilization", systemtime / (elapsed * @processor_count))
            end
            if usertime >= 0
              cpu_utilization = @poller ? @poller.cpu_utilization : (usertime / (elapsed * @processor_count))
              OneApm::Manager.record_metric("CPU/User Time", usertime)
              OneApm::Manager.record_metric("CPU/User/Utilization", cpu_utilization)
            end
          end
          @last_utime = t.utime
          @last_stime = t.stime
          @last_time = now
        end

        def self.supported_on_this_platform?
          # Process.times on JRuby < 1.7.0 reports wall clock elapsed time, not actual cpu time used
          defined?(JRuby) ? (JRUBY_VERSION >= '1.7.0') : true
        end

      end
    end
  end
end
