# encoding: utf-8

require 'one_apm/collector/sampler'

module OneApm
  module Collector
    module Samplers

      class MemorySampler < OneApm::Collector::Sampler
        named :memory

        def set_poller poller
          @poller = poller
        end

        def platform
          OneApm::Collector::Samplers::MemorySampler.platform
        end

        def poll
          sample = get_sample
          if sample
            OneApm::Manager.record_metric("Memory/Used", sample)
          end
        end

        def get_sample
          begin
            m = @poller.memory 
            if m.nil?
              OneApm::Manager.logger.warn "Unable to get the resident memory for process #{$$}.  Disabling memory sampler."
            end
            return m
          rescue => e
            OneApm::Manager.logger.warn "Unable to get the resident memory for process #{$$}. Disabling memory sampler.", e
            return nil
          end
        end

        def self.supported_on_this_platform?
          defined?(::JRuby) or platform =~ /linux|darwin|freebsd|solaris/
        end

        def self.platform
          if RUBY_PLATFORM =~ /java/
            %x[uname -s].downcase
          else
            RUBY_PLATFORM.downcase
          end
        end

      end
    end
  end
end
