# encoding: utf-8

module OneApm
  module Agent
    module Instrumentation
      module Sinatra
        module Ignorer

          def self.should_ignore?(app, type)
            return false if !app.settings.respond_to?(:oneapm_ignores)

            app.settings.oneapm_ignores[type].any? do |pattern|
              pattern.match(app.request.path_info)
            end
          end

          def oneapm_ignore(*routes)
            set_oneapm_ignore(:routes, *routes)
          end

          def oneapm_ignore_apdex(*routes)
            set_oneapm_ignore(:apdex, *routes)
          end

          def oneapm_ignore_enduser(*routes)
            set_oneapm_ignore(:enduser, *routes)
          end

          private

          def set_oneapm_ignore(type, *routes)
            # Important to default this in the context of the actual app
            # If it's done at register time, ignores end up shared between apps.
            set :oneapm_ignores, Hash.new([]) if !respond_to?(:oneapm_ignores)

            # If we call an ignore without a route, it applies to the whole app
            routes = ["*"] if routes.empty?

            settings.oneapm_ignores[type] += routes.map do |r|
              # Ugly sending to private Base#compile, but we want to mimic
              # exactly Sinatra's mapping of route text to regex
              send(:compile, r).first
            end
          end

        end
      end
    end
  end
end
