# encoding: utf-8

LibraryDetection.defer do
  @name = :sequel

  depends_on do
    defined?(::Sequel)
  end

  depends_on do
    !OneApm::Manager.config[:disable_sequel_instrumentation]
  end

  def supported_sequel_version?
    Sequel.const_defined?( :MAJOR ) &&
      ( Sequel::MAJOR > 3 ||
        Sequel::MAJOR == 3 && Sequel::MINOR >= 37 )
  end

  executes do
    if supported_sequel_version?

      OneApm::Manager.logger.info 'Installing Sequel instrumentation'

      if Sequel::Database.respond_to?(:extension)
        Sequel::Database.extension :oneapm_instrumentation
      else
        OneApm::Manager.logger.warn("Detected Sequel version #{Sequel::VERSION}.")
      end

      Sequel.synchronize{Sequel::DATABASES.dup}.each do |db|
        db.extension :oneapm_instrumentation
      end

      Sequel::Model.plugin :oneapm_instrumentation

    else

      OneApm::Manager.logger.info "Sequel instrumentation requires at least version 3.37.0."

    end

  end

end
