# encoding: utf-8

module OneApm
  module Agent
    module Instrumentation
      module Rails4
        module Errors
          def oneapm_notice_error(exception, custom_params = {})
            filtered_params = (respond_to? :filter_parameters) ? filter_parameters(params) : params
            filtered_params.merge!(custom_params)
            OneApm::Transaction.notice_error( \
                exception, \
                :request => request, \
                :custom_params => filtered_params)
          end
        end
      end
    end
  end
end

LibraryDetection.defer do
  @name = :rails4_error

  depends_on do
    defined?(::Rails) && ::Rails::VERSION::MAJOR.to_i >= 4
  end

  depends_on do
    defined?(::ActionController) && defined?(::ActionController::Base)
  end

  executes do
    OneApm::Manager.logger.info "Installing Rails #{::Rails::VERSION::MAJOR} Error instrumentation"
  end

  executes do
    class ActionController::Base
      include OneApm::Agent::Instrumentation::Rails4::Errors
    end
  end
end
