# encoding: utf-8

module OneApm
  class MetricSpec
    attr_reader   :name, :scope

    # the maximum length of a metric name or metric scope
    OA_MAX_LENGTH = 255
    OA_LENGTH_RANGE = (0...OA_MAX_LENGTH)
    OA_EMPTY_SCOPE = ''.freeze

    def initialize(metric_name='', metric_scope=nil)
      if metric_name.to_s.length > OA_MAX_LENGTH
        @name = metric_name.to_s[OA_LENGTH_RANGE]
      else
        @name = metric_name.to_s
      end

      if metric_scope
        if metric_scope.to_s.length > OA_MAX_LENGTH
          @scope = metric_scope.to_s[OA_LENGTH_RANGE]
        else
          @scope = metric_scope.to_s
        end
      else
        @scope = OA_EMPTY_SCOPE
      end
    end

    def ==(o)
      self.eql?(o)
    end

    def eql? o
      @name == o.name && @scope == o.scope
    end

    def hash
      @name.hash ^ @scope.hash
    end

    # return a new metric spec if the given regex
    # matches the name or scope.
    def sub(pattern, replacement, apply_to_scope = true)
      OneApm::Manager.logger.warn("The sub method on metric specs is deprecated") rescue nil
      return nil if name !~ pattern &&
       (!apply_to_scope || scope.nil? || scope !~ pattern)
      new_name = name.sub(pattern, replacement)[OA_LENGTH_RANGE]

      if apply_to_scope
        new_scope = (scope && scope.sub(pattern, replacement)[OA_LENGTH_RANGE])
      else
        new_scope = scope
      end

      self.class.new new_name, new_scope
    end

    def to_s
      return name if scope.empty?
      "#{name}:#{scope}"
    end

    def inspect
      "#<OneApm::MetricSpec '#{name}':'#{scope}'>"
    end

    def to_json(*a)
      {'name' => name,
      'scope' => scope}.to_json(*a)
    end

    def <=>(o)
      namecmp = self.name <=> o.name
      return namecmp if namecmp != 0
      return (self.scope || '') <=> (o.scope || '')
    end
  end
end
