# encoding: utf-8
require 'one_apm/support/rename_rules_engine/match_expression/engine'
require 'one_apm/support/rename_rules_engine/rename/engine'

module OneApm
  module Support
    class RenameRulesEngine
      class ExternalRenameRule
        OA_RENAME_SEPARATOR = '/'.freeze

        attr_accessor :hash_value, :rules

        def initialize(hash={})
          @hash_value = hash['hash_value']
          rules  = hash['rules'] || []
          rules  = rules.select { |r| r['type'] == "remote" }
          rules  = rules.sort_by { |r| r['priority'] }
          rules  = rules.map { |r| Rule.new(r) }
          rules  = rules.reverse
          @rules = rules
        end

        def rename(request)
          selected_rule = select_rule(request)
          OneApm::Manager.logger.debug "request", request.inspect
          OneApm::Manager.logger.debug "selected_rule", selected_rule.inspect
          res = if selected_rule.nil?
                  default_rename(request)
                else
                  selected_rule.apply(request)
                end
          "External/HTTP/#{request.host}:#{request.port}/#{res}"
        end

        def default_rename(request)
          path = request.uri.path.split(OA_RENAME_SEPARATOR)
          path.delete('')
          if path.size <= 1
            "#{path.first}/"
          else
            "#{path[0..1].join(OA_RENAME_SEPARATOR)}"
          end
        end

        def select_rule(request)
          OneApm::Manager.logger.debug "@rules", @rules.inspect
          @rules.select { |r| r.enable?&&r.should_rename?(request) }.first
        end

      end

      class Rule
        OA_RENAME_SEPARATOR = '/'.freeze
        attr_accessor :replacement, :state, :ignore, :priority, :match_expressions, :rename_rules

        def initialize(hash)
          @replacement = hash['replacement']
          @state = hash['state'].to_i
          @ignore = hash['ignore']
          @priority = hash['priority'].to_i
          @match_expressions = hash['match_expression'].map { |match_expression| MatchExpression::Engine.create_match_expression(match_expression) }
          @rename_rules = hash['rename'].map { |rename| Rename::Engine.create_rename_rules(rename) }
        end

        def should_rename?(request)
          match_expressions.map do |match_expression|
            match_expression.match?(request)
          end.all?
        end

        def enable?
          !ignore && state == 0
        end

        def apply(request)
          rename_rules.map do |rename_rule|
            rename_rule.segment(request)
          end.join(OA_RENAME_SEPARATOR)
        end
      end

    end
  end
end
